@@ -10,7 +10,7 @@ from EnvironmentVariable            import *
 from FileExists                     import *
 from InPath                         import *
 from Directory                      import *
-from PersistentDirectory            import *
+#from PersistentDirectory            import *
 from Platform                       import *
 from Username                       import *
 from URL                            import *
@@ -67,7 +67,7 @@ from TextLine                       import *
 from Mail                           import *
 from WorkSpace                      import *
 from FileAccess                     import *
-from Registry                       import *
+#from Registry                       import *
 #from InstalledPackage               import *
 from FileTransferSpeed              import *
 #from RemoteGroup                    import *
@@ -75,12 +75,17 @@ from CommandLineSwitch              import *
 
 from AtomUtils                      import *
 #import Package,Installation,Cat
-import Package,Cat,Already,Watch,SaveRestore,Md5sumCheck
+import Package,Cat,Already,Watch,SaveRestore,Md5sumCheck,Registry,AbsPath
+import GlibcVersion
 
 from AtomDoc import *
 
 catd = AtomDoc('cat','path')
 
+def absPath(path='.'):
+	strArg(path,'absPath')
+	return AbsPath.AbsPath(path)
+
 def md5check(path,md5string):
 	strArg(path,'md5check')
 	strArg(md5string,'md5check')
@@ -147,10 +152,10 @@ def downloadTime(path,maximumSeconds):
 	argchk(maximumSeconds,FloatType,'timedDownload')
 	return DownloadTime(path,maximumSeconds)
 
-def fileCopyMinimumMegsPerSecond(path,minimumMegsPerSecond,fileSizeInMegs=1000):
+def fileCopyMinimumMegsPerSecond(minimumMegsPerSecond,fileSizeInMegs=1000,path='.'):
 	strArg(path,'fileTransferMinimumMegsPerSecond')
 	intArg(fileSizeInMegs,'fileTransferMinimumMegsPerSecond')
-	argchk(minimumMegsPerSecond,FloatType,'fileTransferMinimumMegsPerSecond')
+#	argchk(minimumMegsPerSecond,FloatType,'fileTransferMinimumMegsPerSecond')
 	return FileTransferSpeedMinimum(path,minimumMegsPerSecond,fileSizeInMegs)
 
 #def alreadyInstalled(packageString):
@@ -164,12 +169,12 @@ def register(symbolicName,cacheUrl,infoString='',infoUrl='',contactString='',con
 	strArg(infoUrl,           'registry')
 	strArg(contactString,     'registry')
 	strArg(contactEmail,      'registry')
-	r = Register(symbolicName,cacheUrl,infoString,infoUrl,contactString,contactEmail)
-	r.satisfy().require()
+	r = Registry.Register(symbolicName,cacheUrl,infoString,infoUrl,contactString,contactEmail)
+#	r.satisfy().require()
 	return r
 #	return Register(symbolicName,cacheUrl,infoString,infoUrl,contactString,contactEmail)
 
-def workspace(name,environmentVariable,minmegs,owner=getusername(),options='ownerWrite temporary'):
+def workspace(name,environmentVariable,minmegs,owner=getusername(),options='ownerWrite permanent'):
 	strArg(name,'workspace')
 	strArg(environmentVariable,'workspace')
 	intArg(minmegs,'workspace')
@@ -179,60 +184,176 @@ def workspace(name,environmentVariable,minmegs,owner=getusername(),options='owne
 	return WorkSpace(name,environmentVariable,minmegs,owner,options)
 
 def  isWorldRead   (path,action='on'):
-	strArg(path,'worldRead'); strArg(action,'worldRead')
-	return FileAccess(path,'worldRead',action)
+	strArg(path,'isWorldRead'); strArg(action,'isWorldRead')
+	return HasFileAccess(path,'worldRead',action)
 def  isWorldWrite  (path,action='on'):
-	strArg(path,'worldWrite'); strArg(action,'worldWrite')
-	return FileAccess(path,'worldWrite',action)
+	strArg(path,'isWorldWrite'); strArg(action,'isWorldWrite')
+	return HasFileAccess(path,'worldWrite',action)
 def  isWorldExecute(path,action='on'):
-	strArg(path,'worldExecute'); strArg(action,'worldExecute')
-	return FileAccess(path,'worldExecute',action)
+	strArg(path,'isWorldExecute'); strArg(action,'isWorldExecute')
+	return HasFileAccess(path,'worldExecute',action)
 def  isGroupRead   (path,action='on'):
-	strArg(path,'groupRead'); strArg(action,'groupRead')
-	return FileAccess(path,'groupRead',action)
+	strArg(path,'isGroupRead'); strArg(action,'isGroupRead')
+	return HasFileAccess(path,'groupRead',action)
 def  isGroupWrite  (path,action='on'):
-	strArg(path,'groupWrite'); strArg(action,'groupWrite')
-	return FileAccess(path,'groupWrite',action)
+	strArg(path,'isGroupWrite'); strArg(action,'isGroupWrite')
+	return HasFileAccess(path,'groupWrite',action)
 def  isGroupExecute(path,action='on'):
-	strArg(path,'groupExecute'); strArg(action,'groupExecute')
-	return FileAccess(path,'groupExecute',action)
+	strArg(path,'isGroupExecute'); strArg(action,'isGroupExecute')
+	return HasFileAccess(path,'groupExecute',action)
 def  isOwnerRead   (path,action='on'):
-	strArg(path,'ownerRead'); strArg(action,'ownerRead')
-	return FileAccess(path,'ownerRead',action)
+	strArg(path,'isOwnerRead'); strArg(action,'isOwnerRead')
+	return HasFileAccess(path,'ownerRead',action)
 def  isOwnerWrite  (path,action='on'):
-	strArg(path,'ownerWrite'); strArg(action,'ownerWrite')
-	return FileAccess(path,'ownerWrite',action)
+	strArg(path,'isOwnerWrite'); strArg(action,'isOwnerWrite')
+	return HasFileAccess(path,'ownerWrite',action)
 def  isOwnerExecute(path,action='on'):
-	strArg(path,'ownerExecute'); strArg(action,'ownerExecute')
+	strArg(path,'isOwnerExecute'); strArg(action,'isOwnerExecute')
+	return HasFileAccess(path,'ownerExecute',action)
+
+def  isWorldReadable   (path,action='on'):
+	strArg(path,'isWorldRead'); strArg(action,'isWorldRead')
+	return HasFileAccess(path,'worldRead',action)
+def  isWorldWriteable  (path,action='on'):
+	strArg(path,'isWorldWrite'); strArg(action,'isWorldWrite')
+	return HasFileAccess(path,'worldWrite',action)
+def  isWorldWritable  (path,action='on'):
+	strArg(path,'isWorldWrite'); strArg(action,'isWorldWrite')
+	return HasFileAccess(path,'worldWrite',action)
+def  isWorldExecuteable(path,action='on'):
+	strArg(path,'isWorldExecute'); strArg(action,'isWorldExecute')
+	return HasFileAccess(path,'worldExecute',action)
+def  isWorldExecutable(path,action='on'):
+	strArg(path,'isWorldExecute'); strArg(action,'isWorldExecute')
+	return HasFileAccess(path,'worldExecute',action)
+def  isGroupReadable   (path,action='on'):
+	strArg(path,'isGroupRead'); strArg(action,'isGroupRead')
+	return HasFileAccess(path,'groupRead',action)
+def  isGroupWriteable  (path,action='on'):
+	strArg(path,'isGroupWrite'); strArg(action,'isGroupWrite')
+	return HasFileAccess(path,'groupWrite',action)
+def  isGroupWritable  (path,action='on'):
+	strArg(path,'isGroupWrite'); strArg(action,'isGroupWrite')
+	return HasFileAccess(path,'groupWrite',action)
+def  isGroupExecuteable(path,action='on'):
+	strArg(path,'isGroupExecute'); strArg(action,'isGroupExecute')
+	return HasFileAccess(path,'groupExecute',action)
+def  isGroupExecutable(path,action='on'):
+	strArg(path,'isGroupExecute'); strArg(action,'isGroupExecute')
+	return HasFileAccess(path,'groupExecute',action)
+def  isOwnerReadable   (path,action='on'):
+	strArg(path,'isOwnerRead'); strArg(action,'isOwnerRead')
+	return HasFileAccess(path,'ownerRead',action)
+def  isOwnerWriteable  (path,action='on'):
+	strArg(path,'isOwnerWrite'); strArg(action,'isOwnerWrite')
+	return HasFileAccess(path,'ownerWrite',action)
+def  isOwnerWritable  (path,action='on'):
+	strArg(path,'isOwnerWrite'); strArg(action,'isOwnerWrite')
+	return HasFileAccess(path,'ownerWrite',action)
+def  isOwnerExecuteable(path,action='on'):
+	strArg(path,'isOwnerExecute'); strArg(action,'isOwnerExecute')
+	return HasFileAccess(path,'ownerExecute',action)
+def  isOwnerExecutable(path,action='on'):
+	strArg(path,'isOwnerExecute'); strArg(action,'isOwnerExecute')
+	return HasFileAccess(path,'ownerExecute',action)
+	
+def setWorldRead(path,action='on'):
+	strArg(path,  'setWorldReadable')
+	strArg(action,'setWorldReadable')
+	return FileAccess(path,'worldRead',action)
+def setWorldWrite(path,action='on'):
+	strArg(path,  'setWorldWriteable')
+	strArg(action,'setWorldWriteable')
+	return FileAccess(path,'worldWrite',action)
+def setWorldExecute(path,action='on'):
+	strArg(path,  'setWorldExecutable')
+	strArg(action,'setWorldExecutable')
+	return FileAccess(path,'worldExecute',action)
+def setGroupRead(path,action='on'):
+	strArg(path,  'setGroupReadable')
+	strArg(action,'setGroupReadable')
+	return FileAccess(path,'groupRead',action)
+def setGroupWrite(path,action='on'):
+	strArg(path,  'setGroupWriteable')
+	strArg(action,'setGroupWriteable')
+	return FileAccess(path,'groupWrite',action)
+def setGroupExecute(path,action='on'):
+	strArg(path,  'setGroupExecutable')
+	strArg(action,'setGroupExecutable')
+	return FileAccess(path,'groupExecute',action)
+def setOwnerRead(path,action='on'):
+	strArg(path,  'setOwnerReadable')
+	strArg(action,'setOwnerReadable')
+	return FileAccess(path,'ownerRead',action)
+def setOwnerWrite(path,action='on'):
+	strArg(path,  'setOwnerWriteable')
+	strArg(action,'setOwnerWriteable')
+	return FileAccess(path,'ownerWrite',action)
+def setOwnerExecute(path,action='on'):
+	strArg(path,  'setOwnerExecutable')
+	strArg(action,'setOwnerExecutable')
 	return FileAccess(path,'ownerExecute',action)
 	
-def setWorldReadable(path):
-	strArg(path,'worldReadable')
-	return HasFileAccess(path,'worldRead','on')
-def setWorldWriteable(path):
-	strArg(path,'worldWriteable')
-	return HasFileAccess(path,'worldWrite','on')
-def setWorldExecutable(path):
-	strArg(path,'worldExecutable')
-	return HasFileAccess(path,'worldExecute','on')
-def setGroupReadable(path):
-	strArg(path,'groupReadable')
-	return HasFileAccess(path,'groupRead','on')
-def setGroupWriteable(path):
-	strArg(path,'groupWriteable')
-	return HasFileAccess(path,'groupWrite','on')
-def setGroupExecutable(path):
-	strArg(path,'groupExecutable')
-	return HasFileAccess(path,'groupExecute','on')
-def setOwnerReadable(path):
-	strArg(path,'ownerReadable')
-	return HasFileAccess(path,'ownerRead','on')
-def setOwnerWriteable(path):
-	strArg(path,'ownerWriteable')
-	return HasFileAccess(path,'ownerWrite','on')
-def setOwnerExecutable(path):
-	strArg(path,'ownerExecutable')
-	return HasFileAccess(path,'ownerExecutable','on')
+def setWorldReadable(path,action='on'):
+	strArg(path,  'setWorldReadable')
+	strArg(action,'setWorldReadable')
+	return FileAccess(path,'worldRead',action)
+def setWorldWriteable(path,action='on'):
+	strArg(path,  'setWorldWriteable')
+	strArg(action,'setWorldWriteable')
+	return FileAccess(path,'worldWrite',action)
+def setWorldWritable(path,action='on'):
+	strArg(path,  'setWorldWriteable')
+	strArg(action,'setWorldWriteable')
+	return FileAccess(path,'worldWrite',action)
+def setWorldExecuteable(path,action='on'):
+	strArg(path,  'setWorldExecutable')
+	strArg(action,'setWorldExecutable')
+	return FileAccess(path,'worldExecute',action)
+def setWorldExecutable(path,action='on'):
+	strArg(path,  'setWorldExecutable')
+	strArg(action,'setWorldExecutable')
+	return FileAccess(path,'worldExecute',action)
+def setGroupReadable(path,action='on'):
+	strArg(path,  'setGroupReadable')
+	strArg(action,'setGroupReadable')
+	return FileAccess(path,'groupRead',action)
+def setGroupWriteable(path,action='on'):
+	strArg(path,  'setGroupWriteable')
+	strArg(action,'setGroupWriteable')
+	return FileAccess(path,'groupWrite',action)
+def setGroupWritable(path,action='on'):
+	strArg(path,  'setGroupWriteable')
+	strArg(action,'setGroupWriteable')
+	return FileAccess(path,'groupWrite',action)
+def setGroupExecuteable(path,action='on'):
+	strArg(path,  'setGroupExecutable')
+	strArg(action,'setGroupExecutable')
+	return FileAccess(path,'groupExecute',action)
+def setGroupExecutable(path,action='on'):
+	strArg(path,  'setGroupExecutable')
+	strArg(action,'setGroupExecutable')
+	return FileAccess(path,'groupExecute',action)
+def setOwnerReadable(path,action='on'):
+	strArg(path,  'setOwnerReadable')
+	strArg(action,'setOwnerReadable')
+	return FileAccess(path,'ownerRead',action)
+def setOwnerWriteable(path,action='on'):
+	strArg(path,  'setOwnerWriteable')
+	strArg(action,'setOwnerWriteable')
+	return FileAccess(path,'ownerWrite',action)
+def setOwnerWritable(path,action='on'):
+	strArg(path,  'setOwnerWriteable')
+	strArg(action,'setOwnerWriteable')
+	return FileAccess(path,'ownerWrite',action)
+def setOwnerExecuteable(path,action='on'):
+	strArg(path,  'setOwnerExecutable')
+	strArg(action,'setOwnerExecutable')
+	return FileAccess(path,'ownerExecute',action)
+def setOwnerExecutable(path,action='on'):
+	strArg(path,  'setOwnerExecutable')
+	strArg(action,'setOwnerExecutable')
+	return FileAccess(path,'ownerExecute',action)
 
 def ownedBy(path,owner=getusername()):
 	strArg(path,'ownedBy')
@@ -355,6 +476,10 @@ def systemVersionGE(text):
 	strArg(text,'systemVersionGE')
 	return SystemVersionGE(text)
 	
+def systemVersionGT(text):
+	strArg(text,'systemVersionGT')
+	return SystemVersionGT(text)
+	
 def systemRelease(text):
 	strArg(text,'systemRelease')
 	return SystemReleaseEQ(text)
@@ -371,6 +496,10 @@ def systemReleaseGE(text):
 	strArg(text,'systemReleaseGE')
 	return SystemReleaseGE(text)
 	
+def systemReleaseGT(text):
+	strArg(text,'systemReleaseGT')
+	return SystemReleaseGT(text)
+	
 def systemReleaseGT(text):
 	strArg(text,'sytsemReleaseGT')
 	return SystemReleaseGT(text)
@@ -380,30 +509,24 @@ def pythonCompilerCompiler(text):
 	return PythonCompiler(text)
 			
 def version(arg):
-	if type(arg)==StringType: text = arg
-	else:                     text = `arg`
-	strArg(text,'version')
-	return Version(text)
+	strArg(arg,'version')
+	return Version(arg)
 	
 def versionLE(arg):
-	if type(arg)==StringType: text = arg
-	else:                     text = `arg`
-	return VersionLE(text)
+	strArg(arg,'versionLE')
+	return VersionLE(arg)
 	
 def versionLT(arg):
-	if type(arg)==StringType: text = arg
-	else:                     text = `arg`
-	return VersionLT(text)
+	strArg(arg,'versionLT')
+	return VersionLT(arg)
 	
 def versionGE(arg):
-	if type(arg)==StringType: text = arg
-	else:                     text = `arg`
-	return VersionGE(text)
+	strArg(arg,'versionGE')
+	return VersionGE(arg)
 	
 def versionGT(arg):
-	if type(arg)==StringType: text = arg
-	else:                     text = `arg`
-	return VersionGT(text)
+	strArg(arg,'versionGT')
+	return VersionGT(arg)
 	
 def versionTuple(arg,separator='.'):
 	strArg(arg,'versionTuple')
@@ -553,6 +676,30 @@ def gccVersionGE(arg):
 def gccVersionGT(arg):
 	strArg(arg,'gccVersionGT')
 	return GccVersionGT(arg)
+	
+def gccVersion(arg):
+	strArg(arg,'gccVersion')
+	return GccVersion(arg)
+	
+def glibcVersion(arg):
+	strArg(arg,'gccVersion')
+	return GlibcVersion.GlibcVersion(arg)
+	
+def glibcVersionLE(arg):
+	strArg(arg,'gccVersionLE')
+	return GlibcVersion.GlibcVersionLE(arg)
+	
+def glibcVersionLT(arg):
+	strArg(arg,'gccVersionLT')
+	return GlibcVersion.GlibcVersionLT(arg)
+	
+def glibcVersionGE(arg):
+	strArg(arg,'gccVersionGE')
+	return GlibcVersion.GlibcVersionGE(arg)
+	
+def glibcVersionGT(arg):
+	strArg(arg,'gccVersionGT')
+	return GlibcVersion.GlibcVersionGT(arg)
 
 def gccBinary(path,ver):
 	strArg(path,'gccBinaryEQ')
@@ -641,23 +788,28 @@ def perlVersionGT(arg):
 
 def linuxKernel(arg):
 	strArg(arg,'linuxKernel')
-	return LinuxKernel(arg)
+#	return LinuxKernel(arg)
+	return AND(PlatformGE('Linux'),SystemReleaseEQ(arg))
 	
 def linuxKernelLE(arg):
 	strArg(arg,'linuxKernelLE')
-	return LinuxKernelLE(arg)
+#	return LinuxKernelLE(arg)
+	return AND(PlatformGE('Linux'), SystemReleaseLE(arg))
 	
 def linuxKernelLT(arg):
 	strArg(arg,'linuxKernelLT')
-	return LinuxKernelLT(arg)
+#	return LinuxKernelLT(arg)
+	return AND(PlatformGE('Linux'), SystemReleaseLT(arg))
 	
 def linuxKernelGE(arg):
 	strArg(arg,'linuxKernelGE')
-	return LinuxKernelGE(arg)
+#	return LinuxKernelGE(arg)
+	return AND(PlatformGE('Linux'), SystemReleaseGE(arg))
 	
 def linuxKernelGT(arg):
 	strArg(arg,'linuxKernelGT')
-	return LinuxKernelGT(arg)
+#	return LinuxKernelGT(arg)
+	return AND(PlatformGE('Linux'), SystemReleaseGT(arg))
 
 def pacmanVersion(arg):
 	strArg(arg,'pacmanVersion')
@@ -705,10 +857,21 @@ def envHasValueTemp(env,value):
 	strArg(value,'envHasValueTemp')
 	return EnvHasValue(env,value)
 
-def alias(aliasFrom,aliasTo):
-	strArg(aliasFrom,'alias')
-	strArg(aliasTo,  'alias')
-	return Alias(aliasFrom,aliasTo)
+def setenvShell(env,command):
+	strArg(env,'setenvShell')
+	strArg(command,'setenvShell')
+	return SetenvShell(env,command)
+	
+def setenvShellTemp(env,command):
+	strArg(env,'setenvShellTemp')
+	strArg(command,'setenvShellTemp')
+	return SetenvShellTemp(env,command)
+
+#
+#def alias(aliasFrom,aliasTo):
+#	strArg(aliasFrom,'alias')
+#	strArg(aliasTo,  'alias')
+#	return Alias(aliasFrom,aliasTo)
 
 def exists(filename):
 	strArg(filename,'exists')
@@ -836,7 +999,9 @@ def shell(command):
 	
 def shellDialogue(command):
 	strArg(command,'shellDialogue')
-	return ShellDialogue(command)
+	sh = ShellDialogue(command)
+	sh.mode = 'compatibility'
+	return sh
 	
 def shellOutputContains(command,contains):
 	strArg(command,'shellOutputContains')
@@ -848,6 +1013,11 @@ def shellOutputLE(command,text):
 	strArg(text,   'shellOutputLE')
 	return ShellOutputLE(command,text)
 	
+def shellOutputLT(command,text):
+	strArg(command,'shellOutputLT')
+	strArg(text,   'shellOutputLT')
+	return ShellOutputLT(command,text)
+	
 def shellOutputEQ(command,text):
 	strArg(command,'shellOutputEQ')
 	strArg(text,   'shellOutputEQ')
@@ -858,6 +1028,11 @@ def shellOutputGE(command,text):
 	strArg(text,   'shellOutputGE')
 	return ShellOutputGE(command,text)
 	
+def shellOutputGT(command,text):
+	strArg(command,'shellOutputGT')
+	strArg(text,   'shellOutputGT')
+	return ShellOutputGT(command,text)
+	
 def uninstallShell(command):
 	strArg(command,'uinstallShell')
 	return UninstallShellCommand(command)
@@ -925,9 +1100,11 @@ def freeMegs(path=''):
 	strArg(path2,'freeMegs')
 	return FreeDiskMegs(path2)
 	
-def freeDisk(freemegs,path='.'):
-	if not type(freemegs) is IntType: abort('Type error in argument to freeDisk.')
-	return FreeMegs(freemegs,path)
+def freeDisk(freemegs,path=''): return freeMegsMinimum(freemegs,path)
+#	
+#def freeDisk(freemegs,path='.'):
+#	if not type(freemegs) is IntType: abort('Type error in argument to freeDisk.')
+#	return FreeMegsMinimum(freemegs,path)
 	
 def copy(copyFrom,copyTo):
 	strArg(copyFrom,'copy'); strArg(copyTo,'copy')
@@ -987,15 +1164,15 @@ def softLink(linkFrom,linkTo):
 	strArg(linkFrom,'softLink'); strArg(linkTo,'softLink')
 	return SoftLink(linkFrom,linkTo)
 
-def textFile(filename,lines):
-	return TextFile(filename,lines)
+def textFile(filename,lines,trans=0):
+	return TextFile(filename,lines,trans)
 
 def download(url):
 	strArg(url,'download')
 	return Download(url)
 	
 def downloaduntarzipwrap(url,env):
-	if tail(url,'.tar.gz') or tail(url,'.tgz') or tail(url,'.tar.Z') or tail(url,'.tar.z'):
+	if tail(url,'.tar.gz') or tail(url,'.tgz') or tail(url,'.tar.Z') or tail(url,'.tar.z') or tail(url,'.tar'):
 		D = DownloadUntarzip(url,env)
 	else:
 		D = Download(url)
@@ -1006,6 +1183,9 @@ def downloadUntar(url,env=''):
 	strArg(env,'downloadUntar')
 	return downloaduntarzipwrap(url,env)
 	
+def downloadUntarZip(url,env=''):
+	return downloadUntarzip(url,env)
+	
 def downloadUntarzip(url,env=''):
 	strArg(url,'downloadUntarzip')
 	strArg(env,'downloadUntarzip')
@@ -1027,8 +1207,8 @@ def path(val,env='PATH',mode='front'):
 	return Path(val,env,mode)
 
 def chownR(directory,username=getusername()):
-	strArg(directory,'chownR'); strArg(username2,'chownR')
-	return ChownR(directory,username2)
+	strArg(directory,'chownR'); strArg(username,'chownR')
+	return ChownR(directory,username)
 
 def runningProcess(processname):
 	strArg(processname,'runningProcess')
@@ -1046,6 +1226,10 @@ def udpPorts(host,rangeStart,rangeEnd=0):
 	argchk(rangeEnd,  IntType,'tcpPorts')
 	return UDPPorts(rangeStart,rangeEnd,host)
 	
+def usePort(port):
+	argchk(port,IntType,'usePort')
+	return UsePortNumber(port)
+	
 def chooseDirectory(enviroName,directory):
 	strArg(enviroName,'chooseDirectory')
 	strArg(directory,'chooseDirectory')
