@@ -9,7 +9,9 @@ class CWD(Environment):
 	title  = 'CWDs'
 	action = 'show cwd'
 	
-	def __init__(self): self.cwd = '- unset -'
+	def __init__(self): 
+		self.acquired = 0
+		self.cwd = '- unset -'
 #-- Set
 	def  equal(self,c): return self.cwd == c.cwd
 	def  str(self):     return self.cwd
@@ -51,11 +53,14 @@ class SetCWD(Environment):
 	title  = 'SetCWD'
 	action = 'set cwd'
 	
-	def __init__(self,location): self.location = location	
+	def __init__(self,location): self.location = location
 	def str(self): return self.location
 	def equal(self,x): return self.location==x.location
 	
-	def satisfied(self): return Reason("CWD has not been set.",not self.acquired)
+	def satisfied (self): 
+		if self.acquired: return Reason()
+		else:             return Reason('CWD not been set.')
+#	def satisfied(self): return Reason("CWD has not been set.",not self.acquired)
 	def satisfiable(self): return Reason()
 	def acquire(self):
 		reason = Reason()
