@@ -3,7 +3,7 @@
 #
 from Base         import *
 from Environment  import *
-import shelve
+#import shelve
 
 class DB(Environment):
 	type   = 'database'
@@ -23,8 +23,9 @@ class DB(Environment):
 	def acquire(self):
 		reason = Reason()
 		self._filename = fullpath(self._filename)
+		abort('error')
 		try:
-			db = shelve.open(self._filename)
+#			db = shelve.open(self._filename)
 			db.close()
 		except (IOError,OSError):
 			reason = Reason("Can't write ["+self._filename+"].")
