@@ -3,7 +3,7 @@
 #
 from Environment import *
 from Execution   import *
-import Home,Package
+#import Home,Package
 import os
 
 class Directory(Environment):
@@ -49,6 +49,8 @@ class Directory(Environment):
 	def retract(self): 
 		reason = Reason()
 		if self.created: 
+			import Home,Package
+
 			home = Home.Home()
 			reason,ps = home.getLocation(self.path)
 			if reason.ok():
@@ -58,7 +60,7 @@ class Directory(Environment):
 					reason = Package.LazyPackage(p._spec).uninstall()
 					if not reason.ok(): break
 				os.chdir(wd)
-			if reason.ok(): reason = execute('chmod -R a+w '+self.path)
+			if reason.ok(): execute('chmod -R a+w '+self.path)
 			if reason.ok(): reason = execute('rm -r -f '+self.path)
 			if reason.ok(): self.created = 0
 		return reason
@@ -70,3 +72,11 @@ class MkDirPersistent(Directory):
 	
 	def retract(self): return Reason()
 	
+
+class PersistentDirectory(Directory):
+	type = 'mkdirPersistent'
+	title = 'Persistent Directories'
+
+#-- Action
+	def retract(self): return Reason()
+	
