@@ -1,6 +1,7 @@
 #
-#	Copyright Saul Youssef, 2004
+#	Copyright Saul Youssef, 2005
 #
+from Base            import *
 from Environment     import *
 from FileGetter      import *
 from Execution       import *
@@ -25,7 +26,7 @@ class Download(Environment):
 		if self._download == '- unset -':
 			s = os.path.basename(self._url)+' from '+self._url
 		else:
-			s = os.path.basename(self._url)+' from '+self._url+' => '+self._download
+			s = os.path.basename(self._url)+' from '+self._url+' => '+self._download 
 		if self._deletable: s = s + ' (deletable)'
 		return s
 	def downfile(self): return os.path.basename(self._url)
@@ -37,29 +38,15 @@ class Download(Environment):
 	def compatible(self,d): return Reason()
 	
 #-- Satisfiable 
-	def satisfied(self):
-		reason = Reason()
-		head,tail = os.path.split(self._url)
-		if os.path.exists(tail):
-			self._download = fullpath(tail)
-		elif self._download=='- unset -':
-			reason = Reason('['+self._url+'] has not been downloaded.')
-		elif not os.path.exists(self._download):
-			if self.getDeletable(): reason = Reason()
-			else:                   reason = Reason('Download ['+self._download+'] is missing.')
-		else:
-			reason = Reason()
-		return reason
+	def satisfied(self): return Reason('['+self._url+'] has not yet been downloaded.',not self.acquired)
 
 	def satisfiable(self):
-		reason = Reason()
-		if 0 and not allow('inaccessible-downloads') and not contains(self._url,'$'):
-			for head,tail in self.headtails():
-				accessor = UniversalAccess(head)
-				reason = Reason("["+head+"] is inaccessible.  Can't download ["+tail+"].",not accessor.access())
-			
-				if reason.ok(): break
-		return reason
+		r = Reason()
+		if not self._url==os.path.expanduser(self._url):
+			r = Reason("Download ["+self._url+"] refers to a user home directory.")
+		elif '$' in self._url and not allow('non-snapshottable-downloads'):
+			r = Reason("Download ["+self._url+"] contains an environment variable and can't be resolved for snapshots or mirrors (see -allow to override).")
+		return r
 
 	def headtails(self):
 		head,tail = os.path.split(os.path.expanduser(self._url))
@@ -73,24 +60,26 @@ class Download(Environment):
 		return hts
 
 	def acquire(self):
-		reason = Reason()
-		for head,tail in self.headtails():
-			try:
-				cwd = os.getcwd()
-				accessor = UniversalAccess(head)
-				if isURL(head): 
-					reason = ask.re('down','OK to download ['+tail+'] from ['+urlparse.urlparse(head)[1]+']?')
-					verbo.log('down','Downloading ['+tail+'] from ['+urlparse.urlparse(head)[1]+']...')
-				else:         
-					reason = ask.re('down','OK to download ['+tail+'] from ['+os.path.basename(head)+']?')
-					verbo.log('down','Downloading ['+tail+'] from ['+os.path.basename(head)+']...')
-				if reason.ok():
-					reason = accessor.getFile(tail)
-			except OSError:
-				reason = Reason("Current directory does not exist.  Can't download.")
-			if reason.ok(): 
-				self._download = fullpath(tail)
-				break
+		reason = self.satisfiable()
+		if reason.ok():
+			for head,tail in self.headtails():
+				try:
+					cwd = os.getcwd()
+					accessor = UniversalAccess(head)
+					if isURL(head): 
+						reason = ask.re('down','OK to download ['+tail+'] from ['+urlparse.urlparse(head)[1]+']?')
+						verbo.log('down','Downloading ['+tail+'] from ['+urlparse.urlparse(head)[1]+']...')
+					else:         
+						reason = ask.re('down','OK to download ['+tail+'] from ['+os.path.basename(head)+']?')
+						verbo.log('down','Downloading ['+tail+'] from ['+os.path.basename(head)+']...')
+					if reason.ok():
+						reason = accessor.getFile(tail)
+				except OSError:
+					reason = Reason("Current directory does not exist.  Can't download.")
+				if reason.ok(): 
+#					self._download = fullpath(tail)
+					self._download = os.path.abspath(os.path.expanduser(tail))
+					break
 		return reason
 		
 	def retract(self):
@@ -107,6 +96,7 @@ class DownloadTime(Environment):
 		self._download = Download(url)
 		self._maxtime = maxtime  # float
 		self._time = 0.0
+		self._url = url
 		
 	def equal(self,x): return self._download==x._download and self._maxtime==x._maxtime
 	def str(self): 
@@ -161,9 +151,40 @@ class DownloadUntarzip(Environment):
 	def satisfied  (self): return self._untar.satisfied()
 
 	def acquire(self):
-		reason = self._download.satisfy()
-		if reason.ok(): 
-			reason = self._untar.satisfy()
-			removeFile(os.path.basename(self._url))
+		nRetry = 0
+#		mRetry = httpGetRetries()
+		mRetry = 0  # turn of these retries as they just confuse users and don't work often enough.  S.Y. Feb. 2006
+		while 1:
+			reason = self._download.satisfy()
+			if reason.ok(): 
+				reason = self._untar.satisfy()
+				removeFile(os.path.basename(self._url))
+			if not reason.ok(): 
+				self.retract()
+			if not reason.ok() and not contains(`reason`,'non-standard character') and nRetry<mRetry:
+				r = ask.re('retry','OK to retry ['+`self`+']?')
+				if r.ok():
+					nRetry = nRetry + 1
+					verbo.log('retry',`self`+' has failed with the following error:')
+					verbo.log('retry',`reason`)
+					verbo.log('retry','Retrying '+`nRetry`+'/'+`mRetry`+'...')
+					verbo.log('retry','Pausing ['+`httpGetPause()`+'] seconds first (see -retry help to adjust this)...')
+					time.sleep(httpGetPause())
+				else:
+					reason = copy.deepcopy(r)
+					break
+			else:
+				if nRetry>0: verbo.log('retry','Retry of ['+`self`+'] has succeeded...')
+				break
+			self._untar._tarpause = nRetry*5
+		self._untar._tarpause = 0
 		return reason
-	def retract(self): return self._untar.restore()
+		
+	def retract(self): 
+		self._download.retract().require()
+		self._download = Download(self._url)
+		r = self._untar.retract()
+		self._untar = Untarzip.Untarzip(os.path.basename(self._url),self._enviro)
+		return r
+		
+	def verify(self): return self._untar.verify()
