@@ -83,10 +83,17 @@ class Environment(Satisfiable,Action,SatisfyOrder, Collectable,Applicable,ApplyS
 	def str(self): abort('Missing str in Environment.')
 	def __repr__(self): return self.type+' '+self.str()
 	def statusStr(self):
-		if        self.lastsat:            s = '[*]'
-		elif  not self.satisfiable().ok(): s = '[X]'
-		else:                              s = '[ ]'
+		if        self.lastsat:                                 s = '[*]'
+		elif      self.lastfail or not self.satisfiable().ok(): s = '[X]'
+		else:                                                   s = '[ ]'
 		return s
+	def satset(self,q):
+		if q:
+			self.lastsat  = 1
+			self.lastfail = 0
+		else:
+			self.lastsat  = 0
+			self.lastfail = 1
 
 #-- Satisfiable
 	def satisfiable(self): return Reason()
@@ -148,7 +155,7 @@ class Environment(Satisfiable,Action,SatisfyOrder, Collectable,Applicable,ApplyS
 	def satisfies(self,x): return self==x
 
 #-- ShellOut
-	def shellOut(self,csh,sh,py,pl): pass
+	def shellOut(self,csh,sh,py,pl,ksh): pass
 	
 #-- Update
 	def updateCheck (self): return Reason()
@@ -282,7 +289,12 @@ class AND(EnvironmentList):
 		self.lastsat  = 0
 		self.lastfail = 0
 		return r
-	def verify     (self): return allReason (self,lambda en: en.verify())
+	def verify     (self): 
+		r = allReason (self,lambda en: en.verify())
+		if not r.ok(): 
+			self.lastsat = 0
+			self.lastfail = 1
+		return r
 		
 #-- SatisfyOrder
 	def satisfies(self,E): 
@@ -295,8 +307,8 @@ class AND(EnvironmentList):
 		if self.acquired and switch('showAcquired'): w.text(' @ ')
 	
 #-- ShellOut
-	def shellOut(self,csh,sh,py,pl):
-		for e in self: e.shellOut(csh,sh,py,pl)
+	def shellOut(self,csh,sh,py,pl,ksh):
+		for e in self: e.shellOut(csh,sh,py,pl,ksh)
 
 class OR(EnvironmentList):
 	type   = 'OR'
@@ -319,7 +331,12 @@ class OR(EnvironmentList):
 
 #-- Satisfiable
 	def satisfied  (self): return existsReason (self,lambda en: en.satisfied  ( ))
-	def verify     (self): return existsReason (self,lambda en: en.verify     ( ))
+	def verify     (self): 
+		r = existsReason (self,lambda en: en.verify     ( ))
+		if not r.ok(): 
+			self.lastsat = 0
+			self.lastfail= 1
+		return r
 	def satisfiable(self): return existsReason (self,lambda en: en.satisfiable( ))
 	def fetch      (self): return existsReason2(self,lambda en: en.fetch      ( ))
 	def setup      (self): return existsReason (self,lambda en: en.setup      ( ))
@@ -339,5 +356,5 @@ class OR(EnvironmentList):
 		w.text(self.type); w.cr()
 		if self.acquired and switch('showAcquired'): w.text(' @ ')
 #-- ShellOut
-	def shellOut(self,csh,sh,py,pl):
-		for e in self: e.shellOut(csh,sh,py,pl)	
+	def shellOut(self,csh,sh,py,pl,ksh):
+		for e in self: e.shellOut(csh,sh,py,pl,ksh)	
