@@ -1,7 +1,7 @@
 #
 #	FileGetter, Copyright Saul Youssef, May 2003.
 #
-import os,shutil,urllib,urlparse,time
+import os,shutil,urllib2,urlparse,time,httplib
 from Base import *
 
 class FileGetter(Set):
@@ -55,7 +55,7 @@ class LocalFileGetter(FileGetter):
 			else:
 				r = Reason("Can't copy ["+full+"] to ["+target_directory+"].  File doesn't exist.")
 		else:
-			r.reason("Can't copy ["+full+"] to ["+target_directory+"]. Directory doesn't exists.")
+			r.reason("Can't copy ["+full+"] to ["+target_directory+"]. Directory doesn't exist.")
 		return r
 			
 	def source(self): return self.directory
@@ -86,7 +86,7 @@ class InternetFileGetter(FileGetter):
 			ntry = ntry + 1
 			r = Reason()
 			try:
-				f = urllib.urlopen(self.url)
+				f = urllib2.urlopen(self.url)
 				line = f.readline()
 				f.close()
 			except (IOError, OSError):
@@ -107,7 +107,7 @@ class InternetFileGetter(FileGetter):
 			ntry = ntry + 1
 			r = Reason()
 			try:
-				f = urllib.urlopen(self.url)
+				f = urllib2.urlopen(self.url)
 			except (IOError, OSError):
 				r = Reason(er(self.filename,self.url))
 			if not r.ok():
@@ -126,7 +126,7 @@ class InternetFileGetter(FileGetter):
 			ntry = ntry + 1
 			r = Reason()
 			try:
-				f = urllib.urlopen(self.url)
+				f = urllib2.urlopen(self.url)
 				file = f.readlines()
 				f.close()
 			except (IOError, OSError):
@@ -156,6 +156,8 @@ class InternetFileGetter(FileGetter):
 					ok = 1; break
 				elif contains(line,'Error 404: File Not Found'):
 					ok = 1; break
+				elif contains(line,'<TITLE>403 Forbidden</TITLE>'):
+					ok = 1; break
 			f.close()
 		except (IOError,OSError):
 			ok = 1
@@ -164,44 +166,70 @@ class InternetFileGetter(FileGetter):
 	def get(self,target_directory):
 		r = Reason()
 		if os.path.isdir(target_directory):
-			urllib.urlcleanup()
 			ntry = 0
-			while ntry<self._maxtry:
-				ntry = ntry + 1
-				r = Reason()
-				try:
-					verbo.log('http','Downloading ['+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+']...')
-				
-					if tail(self.filename,'.pacman') or tail(self.filename,'.html') or not verbo('down'):
-						fname,head = urllib.urlretrieve(self.url,os.path.join(target_directory,self.filename))
-					else:
-						fname,head = urllib.urlretrieve(self.url,os.path.join(target_directory,self.filename),dlMeter)
-						if not switch('q'): flash()
-				
-					if self.errorHtml(fname):
+			errorhtml = 0
+			r = ask.re('http','OK to download ['+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+']?')
+			if r.ok():
+				while ntry<self._maxtry:
+					ntry = ntry + 1
+					try:
+						verbo.log('http','Downloading ['+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+']...')
+					
+						f = urllib2.urlopen(self.url)
+						fname = os.path.join(target_directory,self.filename)
+						g = open(fname,'w')
+						
+						size = 0
+						if verbo('meter'):
+							d = f.info().dict
+							if d.has_key('content-length'): 
+								try:
+									size = int(d['content-length'])
+								except:
+									size = 0
+						block = 10000
+						if verbo('meter') and size>block and size>100000:
+							total = 0
+							while 1:
+								x = f.read(block)
+								total = total + len(x)
+								if x=='': break
+								if size>block: dlMeter(1,total,size)
+								g.write(x)
+							flash()
+						else:
+							x = f.read()
+							g.write(x)
+					
+						if self.errorHtml(fname):
+							errorhtml = 1
+							removeFile(os.path.join(target_directory,self.filename))
+							r.reason('Download of ['+self.filename+'] from ['+self.url+'] has failed.')
+					except (IOError, OSError):
+						verbo.log('http',"Error during download of ["+self.filename+"] from ["+urlparse.urlparse(self.url)[1]+"]...")
 						removeFile(os.path.join(target_directory,self.filename))
-						r.reason('Download of ['+self.filename+'] from ['+self.url+'] has failed.')					
-				except (IOError, OSError):
-					verbo.log('http',"Error during download of ["+self.filename+"] from ["+urlparse.urlparse(self.url)[1]+"]...")
-					removeFile(os.path.join(target_directory,self.filename))
-					r.reason("Error during download of ["+self.filename+"] from ["+urlparse.urlparse(self.url)[1]+"].")
-				except KeyboardInterrupt:
-					verbo.log('http',"Download interrupted by ^C...")
-					removeFile(os.path.join(target_directory,self.filename))
-					raise KeyboardInterrupt
-				except socket.error:
-					verbo.log('http',"Download of ["+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+'] failed.  Connection interrupted...')
-					removeFile(os.path.join(target_directory,self.filename))
-					r.reason("Download of ["+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+'] failed.  Connection interrupted')
-				verbo.log('http','Successfully read ['+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+']...')
-				
-				if not r.ok():
-					if not ntry==self._maxtry:
-						verbo.log('retry','Attempt to get ['+self.url+'] has failed.  Retrying '+`ntry`+'/'+`self._maxtry-1`+'...')
-						time.sleep(self._pause)
-				else:
-					break
-		else: 
+						r.reason("Error during download of ["+self.filename+"] from ["+urlparse.urlparse(self.url)[1]+"].")
+					except KeyboardInterrupt:
+						verbo.log('http',"Download interrupted by ^C...")
+						removeFile(os.path.join(target_directory,self.filename))
+						raise KeyboardInterrupt
+					except httplib.InvalidURL:
+						verbo.log('http',"Invalid http proxy...")
+						r.reason("Invalid http proxy.")
+						abort("Invalid http proxy.")
+					except socket.error:
+						verbo.log('http',"Download of ["+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+'] failed.  Connection interrupted...')
+						removeFile(os.path.join(target_directory,self.filename))
+						r.reason("Download of ["+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+'] failed.  Connection interrupted')
+					verbo.log('http','Successfully read ['+self.filename+'] from ['+urlparse.urlparse(self.url)[1]+']...')
+					
+					if not r.ok() and not errorhtml:
+						if not ntry==self._maxtry:
+							verbo.log('retry','Attempt to get ['+self.url+'] has failed.  Retrying '+`ntry`+'/'+`self._maxtry-1`+'...')
+							time.sleep(self._pause)
+					else:
+						break
+		else:
 			r = Reason("Can't download ["+self.filename+"] to ["+target_directory+"] directory doesn't exist.")
 		return r
 
