@@ -3,8 +3,8 @@
 #
 from Base        import *
 from Environment import *
-import Cache,InstallationCache,InstallationBase,Platform,ShellCommand,LastVersion
-import os
+import Cache,InstallationCache,InstallationBase,Platform,ShellCommand,LastVersion,UniversalCache
+import os,anydbm
 
 class Home(Cache.Cache):
 	_home = Cache.Cache()
@@ -19,7 +19,10 @@ class Home(Cache.Cache):
 				plat = get(os.path.join(pacmanDir,'platform'))
 				if not plat==Platform.Platform():
 					print 'Installation at ['+pac_anchor+'] was made on ['+plat.str()+'].'
-					abort('Attempt to access installation ['+pac_anchor+'] from ['+Platform.Platform().str()+'].')
+					print 'Attempt to access installation ['+pac_anchor+'] from ['+Platform.Platform().str()+'].'
+					import lock
+					lock.unlock()
+					sys.exit(1)
 				try:
 					f = open(os.path.join(pac_anchor,pacmanDir,'anchor'),'r')
 					lines = f.readlines()
@@ -43,11 +46,34 @@ class Home(Cache.Cache):
 					f.close()
 				except (IOError,OSError):
 					print "Error writing to ["+os.path.join(pac_anchor,pacmanDir)+"]."
-					abort("Error writing to ["+os.path.join(pac_anchor,pacmanDir)+"].")
+					removeFile(os.path.join('pac_anchor','pacmanDir','lock'))
+					sys.exit(1)
 				lv = LastVersion.lastVersion()
-				if lv>version: print 'Pacman ['+lv+'] is available at http://physics.bu.edu/pacman/...'
-			
-			Home._home = InstallationCache.InstallationCache(UCL)
+				if verbo('http'): print 'version ['+lv+']'
+				if not lv=='' and lv>version: print 'Pacman ['+lv+'] is available at http://physics.bu.edu/pacman/...'
+				
+			if os.path.exists(os.path.join(pac_anchor,pacmanDir,'version')):
+				try:
+					f = open(os.path.join(pac_anchor,pacmanDir,'version'),'r')
+					lines = f.readlines()
+					f.close()
+				except (IOError,OSError):
+					abort("Can't read ["+os.path.join(pac_anchor,pacmanDir,'version')+"].")
+				if len(lines)>0 and len(lines[0])>1:
+					ver = string.strip(lines[0][:-1])
+					if ver>version: 
+						print "This Pacman installation was created with Pacman version ["+ver+"]."
+						print "You are using Pacman version ["+version+"]."
+						print "Pacman version must be >= ["+ver+"]."
+						removeFile(os.path.join(pac_anchor,pacmanDir,'lock'))
+						sys.exit(1)
+				
+			if os.path.isdir(os.path.join(pac_anchor,pacmanDir,'ii3.1')) or use_old_database:
+				verbo.log('io','Using old database for home installation.')
+				Home._home = InstallationCache.OldInstallationCache(UCL)
+			else:
+				Home._home = UniversalCache.UniversalCache(UCL)._cache
+#				Home._home = InstallationCache.   InstallationCache(UCL)
 			Home._home.init()
 			Home._init = 1
 			self.UCL = Home._home.UCL
@@ -56,11 +82,19 @@ class Home(Cache.Cache):
 			abort('Pacman installation area ['+os.path.join(pac_anchor,pacmanDir)+'] is missing.')
 		r,line = firstLine(os.path.join(pac_anchor,pacmanDir,'version'))
 		if r.ok() and version<string.strip(line) and not contains(line,'pre-release'): 
-			print "Pacman installation was created with Pacman version ["+string.strip(line)+"]."
-			print "You are using Pacman version ["+version+"]."
-			print "You must use Pacman version ["+version+"] or greater for this installation."
+			print "Pacman installation was created with Pacman version ["+string.strip(line)+"] but"
+			print "you are using Pacman version ["+version+"]."
+			print "You must use Pacman version ["+string.strip(line)+"] or greater for this installation."
+			import lock
+			lock.unlock()
 			sys.exit(1)
 #			abort("You must use Pacman version ["+version+"] or greater for this installation.")
+		if not os.path.exists(os.path.join(pac_anchor,pacmanDir,'ie'+version[:3])):
+			print 'This Pacman installation was made with Pacman < '+version[:3]+'.'
+			print 'To use Pacman version ['+version+'], you must re-install.'
+			import lock
+			lock.unlock()
+			sys.exit(1)
 		os.environ['TMP'] = os.path.join(pac_anchor,pacmanDir,'tmp')
 			
 	def save(self):
@@ -81,3 +115,6 @@ class Home(Cache.Cache):
 	def topSpecs      (self          ): return self._home.topSpecs()
 	def refreshParents(self          ): return self._home.refreshParents()
 	def getLocation   (self,location ): return self._home.getLocation(location)
+
+home = Home()
+
