@@ -3,8 +3,8 @@
 #
 from Base        import *
 from Environment import *
-import Cache,Source,CD,CWD,Directory,UniversalCache,Home,EnvironmentVariable,Registry
-import os,copy,string
+import Cache,Source,CD,CWD,Directory,UniversalCache,Home,EnvironmentVariable,Registry,RelPath
+import os,copy,string,time
 
 def reduceCaches(E,caches):
 	if E.type=='AND':
@@ -15,22 +15,36 @@ def reduceCaches(E,caches):
 		if sublistEQ(caches,E._spec.caches,lambda c1,c2: Registry.registry.equiv(c1,c2)):
 			E._spec.caches = caches[:]
 			
-def prepend(e,cache):
-	if e.type=='AND':
-		for ee in e: prepend(ee,cache)
-	elif e.type=='OR':
-		for ee in e: prepend(ee,cache)
-	elif e.type=='lazy package':
-		e._spec.caches.insert(0,cache)
+def equivCachesSpec(specCache,cDB,c):
+	eq = Registry.registry.equiv(specCache,c)
+	if not eq:
+		specType,r,specMirror,specTime,specSnap = UniversalCache.getTypeMirrorTimeSnapshot(specCache)
+		ucl = UniversalCache.UniversalCache(c).UCL
+		if r.ok() and specType=='mirror' and specSnap and cDB.has_key(ucl):
+			cMirror,cTime,cSnap = cDB[ucl]
+			if cSnap and Registry.registry.equiv(cMirror,specMirror) and cTime==specTime:
+				eq = 1
+	return eq
 
-def prepend0(e,cache):
-	if e.type=='AND':
-		for ee in e: prepend0(ee,cache)
-	elif e.type=='OR':
-		for ee in e: prepend0(ee,cache)
-	elif e.type=='lazy package':
-		if len(e._spec.caches)==0: 
-			e._spec.caches.insert(0,cache)
+def satisfiedByCaches(caches,db,pcaches):
+	ok = 1
+	c2 = caches[:]; p2 = pcaches[:]
+	if len(caches)>0:
+		if len(pcaches)>0:
+			ok = equivCachesSpec(caches[0],db,pcaches[0])
+			pmir = UniversalCache.UniversalCache(caches[0]).type=='mirror'
+			if ok:
+				caches. pop(0)
+				pcaches.pop(0)
+				ok = satisfiedByCaches(caches,db,pcaches)
+			elif pmir:
+				pcaches.pop(0)
+				ok = satisfiedByCaches(caches,db,pcaches)
+			else:
+				ok = 0
+		else:
+			ok = 0
+	return ok
 		
 def specParse(text):
 	caches = []
@@ -77,7 +91,8 @@ def cacheMatch(specs,cs):
 	if   len(specs)==0: m = 1
 	elif len(cs   )==0: m = 0
 	else:
-		if Registry.registry.equiv(specs[0],cs[0]):
+#		if Registry.registry.equiv(specs[0],cs[0]):
+		if UniversalCache.equivalentCaches(specs[0],cs[0]):
 			specs.pop(0); cs.pop(0)
 			m = cacheMatch(specs,cs)
 		else:
@@ -98,38 +113,60 @@ class Spec(PreOrder,HtmlOut):
 			self.name = '' # self.name[:-7]
 		if specstring=='.': self.name,self.cacheName = '','.'
 		self.mayBeModifiedBy = ['']
+		self._mirrors = {}
 			
 	def _id (self): return `self.caches,self.name,self.subdirectory,self.file`
-	def __eq__(self,x): return self.caches==x.caches and self.name==x.name and self.file==x.file and \
-		self.subdirectory==x.subdirectory and self.caches==x.caches and \
-		self.location==x.location
+	def __eq__(self,x):  return self.name==x.name and self.file==x.file and \
+			self.subdirectory==x.subdirectory and self.caches==x.caches and self.location==x.location
+			
+	def strip(self,c):
+		spec = copy.deepcopy(self)
+		while len(spec.caches)>0 and Registry.registry.equiv(spec.caches[0],c): spec.caches.pop(0)
+		return spec
+	def equalUp(self,x): return self._equalUp(self.strip(pac_anchor),x.strip(pac_anchor))
+	def _equalUp(self,spec1,spec2):
+		eq = spec1.name==spec2.name and spec1.file==spec2.file and spec1.subdirectory==spec2.subdirectory and spec1.location==spec2.location and len(spec1.caches)==len(spec2.caches)
+		if eq:
+			for i in range(len(spec1.caches)):
+				c1,c2 = spec1.caches[i],spec2.caches[i]
+				eq = c1==c2 or (tail(c1,'.caches') and tail(c2,'.caches'))
+		return eq		
 	def __le__(self,x):
-		q = x.name=='*' or x.name=='' or x.name==self.name
-		if q: q = x.subdirectory==''  or x.subdirectory==self.subdirectory
-		if q: q = x.file==''          or x.file==self.file
+		q = x.name=='*' or x.name=='' or x.name          ==  self.name
+		if q: q = x.subdirectory==''  or x.subdirectory  ==  self.subdirectory
+		if q: q = x.file==''          or x.file          ==  self.file
 		return q
 	def match(self,caches):
-		cs2 = self.caches[:]
-		caches2 = caches [:]
+		cs2 = []
+		for c in self.caches:
+			if not tail(c,'.mirror'): cs2.append(c)
+		caches2 = []
+		for c in caches:
+			if not tail(c,'.mirror'): caches2.append(c)
+#		cs2 = self.caches[:]
+#		caches2 = caches [:]
 		return cacheMatch(cs2,caches2)
-	def satisfiedBy(self,p): return self.satisfiedByR(p).ok()
+	def satisfiedBy (self,p): return self.satisfiedByR(p).ok()
 	def satisfiedByR(self,p):
 		r = Reason()
-		if self.match(p._spec.caches):
+		specCaches = self.caches[:]
+		pCaches    = p._spec.caches[:]
+		if hasattr(p._spec,'_mirrors'): m = p._spec._mirrors
+		else:                           m = {}
+		if satisfiedByCaches(specCaches,m,pCaches):
 			q = p._spec<=self
 			if q:
 				if not self.guard=='':
-#					r,guard = Source.Source(self.guard).comp()
 					r,guard = Source.Source(self.guard,Spec()).comp()
 					if r.ok():
 						if not p._environ.satisfies(guard):
 							r = Reason("Package specification ["+self.str()+"] fails |... requirements of ["+p._spec.str()+"].")
 			else:
-				r = Reason("Package specification ["+self.str()+"] not satisfied by ["+p._spec.str()+"].")
+				r = Reason("Package ["+p._spec.str()+"] does not satisfy ["+self.str()+"].")
 		else:
-			r = Reason("Package specification ["+self.str()+"] does not match the caches of ["+p._spec.str()+"].")
+			r = Reason("Package ["+p._spec.str()+"] does not match the cache specified by ["+self.str()+"].")
 		if verbo('ptest') and not self.name=='' and not self.name=='*':
-			if r.ok(): print "Package specification ["+self.str()+"] satisfied by ["+p._spec.str()+"]."
+			if r.ok(): print "Package ["+p._spec.str()+"] satisfies ["+self.str()+"]."
 			else:      print r
 		return r
 	def __repr__(self): return 'spec: '+self.str()
@@ -152,6 +189,12 @@ class Spec(PreOrder,HtmlOut):
 				else:              comp = fname==self.file
 		return comp
 	def envEqu(self,e): return e.satisfies(Source.Source(self.guard).compile())
+	def cachesPop(self):
+		if self.caches==[]: abort("Error in Pacman.")
+		cache = self.caches.pop(0)
+		if len(self.caches)>0 and RelPath.relPath(self.caches[0]): 
+			self.caches[0] = os.path.join(cache,self.caches[0])
+		return cache
 	def locate(self,e):
 		if self.location=='': 
 			e2 = AND(CWD.SetCWD('.'),EnvironmentVariable.SetenvTemp('PACMAN_INSTALLATION','.'))
@@ -164,17 +207,16 @@ class Spec(PreOrder,HtmlOut):
 		return e2
 	def getAll(self,used=[]):
 		r,ps = Reason(),[]
-		spec = copy.deepcopy(self)
-		
-		if len(spec.caches)==0: r = Reason("Can't find ["+spec.str()+"].")
+		if len(self.caches)==0: r = Reason("Can't find ["+self.str()+"].")
 		if r.ok():
-			c = UniversalCache.UniversalCache(spec.caches.pop(0))
-			r,ps = c.getAll(spec,used)
-			for p in ps:
-				if c.type=='source': prepend0(p._environ,c.UCL)
-				else:                prepend (p._environ,c.UCL)
-				p._spec.caches.insert(0,c.UCL)
-		if len(ps)==0: r = Reason('Package ['+self.str()+'] not found.')
+			spec = copy.deepcopy(self)
+			c0 = spec.caches.pop(0)
+			c = UniversalCache.UniversalCache(c0)
+			r,qs = c.getAll(spec,used)
+			ps = []
+			for q in qs: 
+#				q._spec.caches.insert(0,Registry.registry.short(c0))
+				ps.append(q)
 		return r,ps
 	def get(self):
 		r,ps = self.getAll([])
@@ -221,16 +263,48 @@ class Package(Environment,Update,Verify):
 		self._lazy = 0
 		self._parents   = []
 		self._modifiers = []
+		self.lastsat = 0
+		self.lastfail= 0
 		
 		self._neutered = 0
 		self._lastSatisfiable = Reason()
 		
 	def _id(self): return `self._spec.name,self._sourceCache,self._spec.subdirectory,self._spec.file`
+	def setParent(self):
+		self.setParentBase(self._environ)
+		self.setParentBase(self._source )
+	def setParentBase(self,e):
+		if   e.type=='AND' or e.type=='OR':
+			for i in range(len(e)): self.setParentBase(e[i])
+		elif e.type=='lazy package':
+			spec = Spec(self._spec.name)
+			e._parent = copy.deepcopy(spec)
 #-- Set
 	def equal(self,p):                     return \
 		self._spec        == p._spec      and \
 		self._environ     == p._environ   and \
 		self._source      == p._source 
+	def upEqual(self,es1,es2):
+		if len(es1)==len(es2):
+			eq = 1
+			for i in range(len(es1)):
+				e1,e2 = es1[i],es2[i]
+				if e1.type=='pacman source code' and e2.type=='pacman source code':
+					pass
+				else:
+					if e1.type=='lazy package' and e2.type=='lazy package':
+						eq = e1._spec.equalUp(e2._spec)
+					else:
+						eq = e1==e2
+				if not eq: 
+					if debug('up'): 
+						print self
+						print e1
+						print e2
+					break
+		else:
+			eq = 0
+		return eq
 	def str(self): return self._spec.str()
 	def location(self):
 		if len(self._environ)>0 and self._environ[0].type=='setcwd':
@@ -245,10 +319,10 @@ class Package(Environment,Update,Verify):
 			update      = self.updateWaiting ().ok()
 			loc         = self.location()
 		else:
-			installed   =     self.lastsat
+			installed   = self.lastsat
 			installable = installed or (self._lastSatisfiable.ok() and not self.lastfail)
-			update      =     self.updateWaiting().ok()
-			loc         =     self.location()
+			update      = self.updateWaiting().ok()
+			loc         = self.location()
 		return installable,installed,update,loc
 	def getEs(self,typename):
 		es = []
@@ -278,17 +352,22 @@ class Package(Environment,Update,Verify):
 			 displayMode(        'src') or \
 			 displayMode(       'file') or \
 			 displayMode(        'loc') or \
+			 displayMode(     'mirror') or \
 			 displayMode(        'rel')  ):
-			s = s + ' ' + os.path.join(self._spec.subdirectory,self._spec.name)+', in cache ['+self._inCache+']'  #+', from ['+self._spec.file+']'
+			 s = s + ' ' + os.path.join(self._spec.subdirectory,self._spec.name)#+', in cache ['+self._inCache+']'  #+', from ['+self._spec.file+']'
 		else:
 			s = s + ' ' + os.path.join(self._spec.subdirectory,self._spec.name)
 			if displayMode(       'up'):
+				cachesTemp = copy.deepcopy(self._spec.caches)
+				if len(cachesTemp)>0: cachesTemp.pop(0)
 				s = s + ', update from ['
 				count = 0
-				for cache in self._spec.caches: 
+#				for cache in self._spec.caches: 
+				for cache in cachesTemp: 
 					count = count + 1
-					if count==len(self._spec.caches): s = s + cache
-					else:                             s = s + cache+':'
+#					if count==len(self._spec.caches): s = s + cache
+					if count==len(cachesTemp): s = s + cache
+					else:                      s = s + cache+':'
 				s = s + ']'
 			if displayMode(       'req'): s = s + ' | ' + self._spec.guard
 			if displayMode(        'in'): s = s + ', in cache ['+self._inCache+']'
@@ -335,6 +414,15 @@ class Package(Environment,Update,Verify):
 			for e in self._environ:
 				if e.type=='option':
 					print indent*' '+'           '+`e`
+		if displayMode('mirror') or displayMode('snap'):
+			if hasattr(self._spec,'_mirrors'):
+				keys = self._spec._mirrors.keys()
+				for key in keys:
+					mirror,t,snapshot = self._spec._mirrors[key]
+					if snapshot:
+						print indent*' '+'           '+'...from cache ['+key+'], a snapshot of ['+mirror+'] taken at ['+time.ctime(t)+'].'
+					else:
+						print indent*' '+'           '+'...from cache ['+key+'], a mirror of ['+mirror+'] last updated ['+time.ctime(t)+'].' 
 		if   displayMode('src'):
 			for line in       self._sourceCode: print indent*' '+'         [src] '+line[:-1]
 		if displayMode('cmp'):
@@ -413,6 +501,9 @@ class Package(Environment,Update,Verify):
 		if not _setupBody.has_key(idd):
 			debug.log('setup','Package ['+self._spec.str()+'] is being setup...')
 			r = self._environ.setup()
+			if not r.ok(): 
+				print r
+				r = Reason("Package ["+self._spec.str()+"] has not been installed.  Can't setup.")
 			_setupBody[idd] = ''
 		else:
 			r = Reason()
@@ -427,7 +518,12 @@ class Package(Environment,Update,Verify):
 		return self.reasonFixup(r,'uninstalled')
 
 #-- Verify
-	def verify(self): return self.reasonFixup(self._environ.verify(),'verified')
+	def verify(self): 
+		r = self.reasonFixup(self._environ.verify(),'verified')
+		if not r.ok():
+			self.lastsat = 0
+			self.lastfail= 1
+		return r
 	def repair(self): return self.satisfy  ()
 
 #-- Install
@@ -438,15 +534,16 @@ class Package(Environment,Update,Verify):
 		if r.ok():
 			if self._neutered or depth<0:
 				pass
-			elif len(self._modifiers)>0:
-				r = allReason(self._modifiers,lambda spec: LazyPackage(spec).uninstall())
-			else:
+			elif self.lastsat==1 or self.lastfail==1:
 				if self.lastsat and len(self.getEs('uninstall shell command'))>0: self.setup()
-				r = self.uninstallBody(depth)
-				if not switch('update'): self.uninstallSet()
-			
-			self.lastsat  = 0
-			self.lastfail = 0
+				self.lastsat  = 0
+				self.lastfail = 0
+				
+				if len(self._modifiers)>0: r = allReason(self._modifiers,lambda spec: LazyPackage(spec).uninstall())
+				
+				if r.ok():
+					r = self.uninstallBody(depth)
+					if not switch('update'): self.uninstallSet()
 			
 			if r.ok(): 
 				if verbo('pac'): print 'Package ['+self._spec.str()+'] successfully uninstalled...'
@@ -504,13 +601,17 @@ class Package(Environment,Update,Verify):
 #-- Update
 	def updateCheck(self):
 		self._update = OR()
-		r,p = self._spec.get()
+
+#--  S.Y. Feb. 20, 2006
+		spec2 = copy.deepcopy(self._spec)
+		if len(spec2.caches)>0: spec2.caches.pop(0)
+		r,p = spec2.get()
+#		r,p = self._spec.get()
 		
 		verbo.log('up-check','Checking for update of ['+self._spec.str()+']...')
-		if len(self._spec.caches)>0 and tail(self._spec.caches[0],'.snap'):
-			verbo.log('up-check','Package ['+self._spec.str()+'] installed from a snapshot - no need to check update...')
-		elif r.ok():
-			if not self._source==p._source:
+		if r.ok():
+#			if not self._source==p._source:
+			if not self.upEqual(self._source,p._source):
 				verbo.log('up','Update of ['+self._spec.str()+'] found...')
 				self._update           = p._environ
 				self._updateSourceCode = p._sourceCode[:]
@@ -523,8 +624,6 @@ class Package(Environment,Update,Verify):
 		return self._environ.updateRemove()
 	def updateWaiting(self): return Reason('No update waiting.',self._update == OR())
 	def updateSelf(self):
-#		verbo.log('up','About to update ['+self._spec.str()+']...')
-#		r = ask.re('up','OK to update ['+self._spec.str()+']?')
 		r = Reason()
 		if r.ok():
 			r = Reason("Can't update relative package ["+self.str()+"].",self._neutered)
@@ -547,20 +646,32 @@ class Package(Environment,Update,Verify):
 						verbo.log('up',"Can't uninstall ["+self._spec.name+"].  Not updated...")
 				else:
 					r = self._environ.update()
-#					if r.ok(): verbo.log('up','Package ['+self._spec.str()+'] successfully updated...')
-#					else:      verbo.log('up','Package ['+self._spec.str()+'] update attempt has failed...')
 		return r
 	def update(self):
 		r = Reason("Can't update relative package ["+self.str()+"].",self._neutered)
 		if r.ok():
-			if self.updateWaiting(): r = self.updateSelf()
-			if r.ok(): r = self._environ.update()
+			inst = self.lastsat
+			if self.updateWaiting():
+				if not switch('single'): r = self._environ.update()
+				if r.ok(): r = self.updateSelf()
+
+			rr,pp = UniversalCache.UniversalCache('home').get(self._spec)
+			if rr.ok() and inst and not pp.lastsat:
+				self = pp
+				r = self.satisfy()
+#			if self.updateWaiting(): r = self.updateSelf()
+#			if r.ok(): r = self._environ.update()
 		return self.reasonFixup(r,'updated')
 #-- setup
-	def shellOut(self,csh,sh,py,pl): 
-		lines = ['#\n','#-- '+`self`+'\n','#\n']
-		for line in lines: csh.write(line); sh.write(line); py.write(line); pl.write(line)
-		self._environ.shellOut(csh,sh,py,pl)
+	def shellOut(self,csh,sh,py,pl,ksh): 
+		if self.lastsat:
+			idd = self._id()
+			if not _shellOut.has_key(idd):
+				_shellOut[idd] = ''
+				lines = ['#\n','#-- '+`self`+'\n','#\n']
+				for line in lines: 
+					csh.write(line); sh.write(line); py.write(line); pl.write(line); ksh.write(line)
+				self._environ.shellOut(csh,sh,py,pl,ksh)
 
 _updateCheck   = {}
 _updateRemove  = {}
@@ -630,7 +741,8 @@ class LazyPackage(Package):
 		return r
 	def modifyApply(self,p):
 		if not self._parent==Spec():
-			if not (self._parent in p._parents): p._parents.append(self._parent)
+			if not (self._parent in p._parents): 
+				p._parents.append(self._parent)
 			if self._modified:
 				if not (self._parent in p._modifiers): p._modifiers.append(self._parent)
 		_pardb[self._spec._id()] = p._parents  [:]
@@ -700,12 +812,23 @@ class LazyPackage(Package):
 #-- Install
 	def uninstallSet (self        ): return self.check (_uninstallSet, lambda: self.lazyApplyM(lambda x: x.uninstallSet      ())    )
 	def remove       (self,depth=0): return self.check0(_remove,       lambda: self.lazyApplyU(lambda x: x.remove       (depth))[0] )
-	def uninstall    (self,depth=0): return self.check0(_uninstall,    lambda: self.lazyApplyM(lambda x: x.uninstall    (depth))    )
+	def uninstall    (self,depth=0): 
+		c = UniversalCache.UniversalCache('home')
+		r,p = c.get(self._spec)
+		if r.ok(): 
+			return self.check0(_uninstall,    lambda: self.lazyApplyM(lambda x: x.uninstall    (depth))    )
+		else:
+			return Reason()
 	def uninstallBody(self,depth=0): return self.check0(_uninstallBody,lambda: self.lazyApplyM(lambda x: x.uninstallBody(depth))    )
 
 #-- Verify
 	def verify       (self): return self.check (_verify,       lambda: self.lazyApplyU(lambda x: x.verify        ())[0] )
-	def repair       (self): return self.check (_repair,       lambda: self.lazyApplyM(lambda x: x.repair        ())    )
+	def verifySave   (self): return self.check (_verify,       lambda: self.lazyApplyM(lambda x: x.verify        ())    )
+	def repair       (self): 
+		r = self.verifySave()
+		if not r.ok():
+			r = self.check (_repair,       lambda: self.lazyApplyM(lambda x: x.repair        ())    )
+		return r
 	
 #-- Update
 	def updateCheck  (self): return self.check (_updateCheck,  lambda: self.lazyApplyM(lambda x: x.updateCheck   ())    )
@@ -715,4 +838,4 @@ class LazyPackage(Package):
 	def updateWaiting(self): return self.lazyApplyU(lambda x: x.updateWaiting ())[0]
 
 #-- Setup
-	def shellOut(self,csh,sh,py,pl): return self.check(_shellOut,lambda: self.lazyApplyU(lambda x: x.shellOut(csh,sh,py,pl))[0])
+	def shellOut(self,csh,sh,py,pl,ksh): return self.check(_shellOut,lambda: self.lazyApplyU(lambda x: x.shellOut(csh,sh,py,pl,ksh))[0])
