@@ -14,7 +14,7 @@ import string
 # in the list. The list will probably be ordered like:
 #  [redhat-version, kernel-version, linux, unix]
 def findPlatform():
-	base_platform = sys.platform;
+	base_platform = sys.platform
 	if re.match('linux', base_platform, re.IGNORECASE | re.DOTALL):
 		full_linux_version, linux_version = findLinuxVersion()
 		if full_linux_version != "":
@@ -28,6 +28,9 @@ def findPlatform():
 		full_aix_version, aix_version = findAIXVersion();
 		platforms = [full_aix_version, aix_version,
 				base_platform, 'aix', 'unix']
+	elif base_platform == 'darwin':
+		darwin_version = findDarwinVersion();
+		platforms = [darwin_version, base_platform, 'unix']
 	else:
 		platforms = [base_platform, 'unix']
 	return platforms
@@ -35,8 +38,13 @@ def findPlatform():
 # This is used by findPlatform. It figures out which distribution of
 # Linux we're running on. 
 def findLinuxVersion():
-	try: 
-		if os.path.exists('/etc/redhat-release'):
+	try:
+		# It's important to look for rocks-releaes before
+		# the others.  A ROCKS install might have redhat-release
+		# as well.
+		if os.path.exists('/etc/rocks-release'):
+			issue_filename = '/etc/rocks-release'
+		elif os.path.exists('/etc/redhat-release'):
 			issue_filename = '/etc/redhat-release'
 		elif os.path.exists('/etc/SuSE-release'):
 			issue_filename = '/etc/SuSE-release'
@@ -56,17 +64,34 @@ def findLinuxVersion():
 				distro = 'tao'
 			elif re.search('Mandrake', line):
 				distro = 'mandrake'
-			elif re.search('SuSE', line):
+#			elif re.search('SuSE', line):
+			elif re.compile('SuSE', re.IGNORECASE).search(line):
 				distro = 'suse'
-			elif re.search('Fermi', line):
-				distro = 'fermi'
 			elif re.search('Debian', line):
 				distro = 'debian'
 			elif re.search('Fedora', line):
 				distro = 'fedora'
-			elif re.search('Scientific Linux',line):
-				distro = 'SLC'
-
+			elif re.search('Scientific Linux SL',line):
+				distro = 'sl'
+			elif re.search('Scientific Linux CERN',line):
+				distro = 'sl-cern'
+			elif re.search('Scientific Linux IFIC',line):
+				distro = 'sl-ific'
+			elif re.search('BU Linux',line):
+				distro = 'BU'
+#			elif re.search('Scientific Linux Release [\d\.]+ \(Fermi\)',line):
+			elif re.compile('Scientific Linux Release [\d\.]+ \(Fermi\)',re.IGNORECASE).search(line):
+				distro = 'sl-fermi'
+				# Fermi doesn't have . in their version strings
+				line = re.sub(r"(\d)(\d)(\d)", r"\1.\2.\3", line)
+				
+			# This needs to go after sl-fermi until we can make
+			# a better regexp
+			elif re.search('Fermi', line):
+				distro = 'fermi'
+			elif re.search('Rocks', line):
+				distro = 'rocks'
+			
 #			if distro != None:
 #				version_match = re.search(r"\d+\.\d+", line)
 #				if version_match:
@@ -82,6 +107,8 @@ def findLinuxVersion():
 				version_match = re.search(r"\d+\.\d+", line)
 				if version_match:
 					version_string = line[version_match.start():version_match.end()]
+					# -- BU linux puts their version string in a funny place.  S.Y.
+					if distro=='BU' and len(string.split(line,' '))>5: version_string = string.split(line,' ')[5]
 					versions = ('linux-' + distro + '-' + version_string,
 					   'linux-'+distro)
 				else:
@@ -96,7 +123,7 @@ def findLinuxVersion():
 
 	except IOError:
 		versions = ("", "")
-		
+
 	return versions
 
 def findAIXVersion():
@@ -109,6 +136,19 @@ def findAIXVersion():
 		versions = ("", "")
 	return versions
 
+def findDarwinVersion():
+	try:
+		release = string.rstrip(os.popen('sw_vers -productVersion').readlines()[0])
+		version_match = re.search(r"\d+\.\d+", release)
+		if version_match:
+			version = release[version_match.start():version_match.end()]
+			versions = ('darwin-' + version)
+		else:
+			versions = ("")
+	except IOError:
+		versions = ("", "")
+	return versions
+
 # This class is like a simple dictionary.
 # Unlike the normal dictionary, it can take a list of indices.
 # It searches them in order until it finds one that matches.
