@@ -6,20 +6,26 @@ import pickle
 
 def registry0():
 	reg = {}
-	reg['Dartmouth'] = Register('Dartmouth',  'http://grid.dartmouth.edu/pacman/','Dartmouth','Dartmouth College','James E. Dobson','James.E.Dobson@Dartmouth.EDU')
-	reg['ATLAS']     = Register(    'ATLAS',  'http://atlas.web.cern.ch/Atlas/GROUPS/SOFTWARE/OO/pacman/cache/',                      'The Atlas Experiment, CERN',  'http://atlas.web.cern.ch/Atlas/GROUPS/SOFTWARE/OO/sit/Distribution/', 'Christian Arnault', 'arnault@lal.in2p3.fr')
-	reg['Pacman']    = Register(   'Pacman',  'http://physics.bu.edu/pacman/sample_cache/',                       'Pacman Headquarters, Boston University',  'http://physics.bu.edu/~youssef/pacman/',  'Saul Youssef',                        'youssef@bu.edu')
-	reg['Demo']      = Register(     'Demo',  'http://physics.bu.edu/pacman/demo/',                       'Pacman Demos',  'http://physics.bu.edu/~youssef/pacman/',  'Saul Youssef',                        'youssef@bu.edu')
-	reg['BU']        = Register(       'BU',  'http://atlas.bu.edu/caches/',                            'Atlas Tier 2, Boston University',  'http://physics.bu.edu/~usatlas/',         'Saul Youssef',       'http://physics.bu.edu/~youssef/')
-	reg['BU-ATLAS']  = Register( 'BU-ATLAS',  'http://atlas.bu.edu/caches/',                            'Atlas Tier 2, Boston University',  'http://physics.bu.edu/~usatlas/',         'Saul Youssef',       'http://physics.bu.edu/~youssef/')
-	reg['VDT']       = Register(      'VDT',  'http://www.cs.wisc.edu/vdt/vdt_cache/',                  'Virtual Data Toolkit, University of Wisconson',  'http://www.lsc-group.phys.uwm.edu/vdt/',  'Alain Roy',                          'roy@cs.wisc.edu')
-	reg['BNL-PAS']   = Register(  'BNL-PAS',  'http://www.usatlas.bnl.gov/computing/software/pacman/PAS-cache/','Atlas Software, Brookhaven National Laboratory',  'http://www.usatlas.bnl.gov/atlas_psc/',   'Wensheng Deng',                        'wdeng@bnl.gov')
-	reg['IU']        = Register(       'IU',  'http://iuatlas.physics.indiana.edu/pacman/IUcache/',                                  'Atlas at Indiana University',  'http://www.indiana.edu/',                 'Leigh Grundhoefer',  'leighg@chestnut.ucs.indiana.edu')
+	reg['Dartmouth'] = Register('Dartmouth',  'http://grid.dartmouth.edu/pacman/',                                                                     'Dartmouth',  'Dartmouth College',                       'James E. Dobson',                       'James.E.Dobson@Dartmouth.EDU')
+	reg['OU']        = Register(       'OU',  'http://www-hep.nhn.ou.edu/pacman-cache/',                                                  'University of Oklahoma',  'http://www-hep.nhn.ou.edu/',                                             'Horst Severini',        'hs@nhn.ou.edu')
+	reg['ATLAS']     = Register(    'ATLAS',  'http://atlas.web.cern.ch/Atlas/GROUPS/SOFTWARE/OO/pacman/slc3/cache/',                 'The Atlas Experiment, CERN',  'http://atlas.web.cern.ch/Atlas/GROUPS/SOFTWARE/OO/sit/Distribution/', 'Christian Arnault', 'arnault@lal.in2p3.fr')
+	reg['CMT']       = Register(      'CMT',  'http://www.cmtsite.org/pacman/cache/',                                                                        'CMT',  'http://www.cmtsite.org/', 'Christian Arnault', 'arnault@lal.in2p3.fr')
+	reg['Pacman']    = Register(   'Pacman',  'http://physics.bu.edu/pacman/sample_cache/',                               'Pacman Headquarters, Boston University',  'http://physics.bu.edu/~youssef/pacman/',  'Saul Youssef',                        'youssef@bu.edu')
+	reg['Demo']      = Register(     'Demo',  'http://physics.bu.edu/pacman/demo/',                                                                 'Pacman Demos',  'http://physics.bu.edu/pacman/',           'Saul Youssef',                        'youssef@bu.edu')
+	reg['BU']        = Register(       'BU',  'http://atlas.bu.edu/caches/',                                                     'Atlas Tier 2, Boston University',  'http://physics.bu.edu/~usatlas/',         'Saul Youssef',       'http://physics.bu.edu/~youssef/')
+	reg['BNL']       = Register(      'BNL',  'http://www.usatlas.bnl.gov/computing/cache/',   'Brookhaven National Laboratory',  'http://www.usatlas.bnl.gov/',  'Alex Undrus',       'undrus@bnl.gov')
+	reg['VDT']       = Register(      'VDT',  'http://vdt.cs.wisc.edu/vdt_cache/',                             'Virtual Data Toolkit, University of Wisconson',  'http://vdt.cs.wisc.edu/',  'Alain Roy and Nate Mueller',                          'roy@cs.wisc.edu')
+	reg['OSG']       = Register(      'OSG',  'http://software.grid.iu.edu/pacman/',                                  'Open Science Grid',  'http://www.opensciencegrid.org/',                 'Leigh Grundhoefer',  'leighg@chestnut.ucs.indiana.edu')
 	reg['UTA']       = Register(      'UTA',  'http://heppc12.uta.edu/pacman/',                                          'Atlas, University of Texas at Arlington',  'http://heppc12.uta.edu/atlas/',           'Kaushik De',                         'kaushik@uta.edu')
-	reg['UCHEP']     = Register(    'UCHEP',  'http://grid.uchicago.edu/caches/',                                               'HEP at the University of Chicago',  'http://grid.uchicago.edu/atlaschimera',   'Yuri Smirnov',              'smirnov@hep.uchicago.edu')
-	reg['GCL']       = Register(      'GCL',  'http://grid.uchicago.edu/caches/gcl/',                              'Grid Component Library, University of Chicago',  'http://grid.uchicago.edu/gcl/',           'Yuri Smirnov',              'smirnov@hep.uchicago.edu')
-	reg['ISI']       = Register(      'ISI',  'http://www.isi.edu/~olomu/rls/rls-cache/',                         'RLS at the Information Sciences Institute, USC',  'http://www.isi.edu/',                     'Nosa Olomu',                            'nosa@isi.edu')
+	reg['UCHEP']     = Register(    'UCHEP',  'http://grid.uchicago.edu/caches/',                                               'HEP at the University of Chicago',  'http://grid.uchicago.edu/atlaschimera',   'Marco Mambelli',              'marco@hep.uchicago.edu')
+	reg['GCL']       = Register(      'GCL',  'http://grid.uchicago.edu/caches/gcl/',                              'Grid Component Library, University of Chicago',  'http://grid.uchicago.edu/gcl/',           'Marco Mambelli',              'marco@hep.uchicago.edu')
 	reg['iVDGL']     = Register(    'iVDGL',  'http://hep.uchicago.edu/ivdgl/',                                       'International Virtual Data Grid Laboratory',  'http://www.ivdgl.org/',                   'Rob Gardner',                   'rwg@hep.uchicago.edu')
+        reg['GridCat']   = Register(  'GridCat',  'http://www.ivdgl.org/gridcat/GridCat/',                                                     'Grid Cataloging System', 'http://www.ivdgl.org/',                   'Bockjoo Kim',                   'bockjoo@phys.ufl.edu')
+	reg['LDR']       = Register(      'LDR',  'http://www.lsc-group.phys.uwm.edu/LDR/cache/',                                            'Lightweight Data  Replicator', 'http://www.lsc-group.phys.uwm.edu/LDR',   'Scott Koranda',        'skoranda@gravity.phys.uwm.edu')
+	reg['STAR']      = Register(     'STAR',  'http://www.star.bnl.gov/STAR/comp/ofl/pacman/caches/', 'The STAR Experiment at RHIC, BNL','http://www.star.bnl.gov/','Valeri Fine','fine@bnl.gov')
+        reg['TeraGrid']  = Register( 'TeraGrid',  'http://software.teragrid.org/pacman/','NSF TeraGrid','http://www.teragrid.org/','JP Navarro','navarro@mcs.anl.gov')
+	reg['IEPM-BW']   = Register(  'IEPM-BW',  'http://www.slac.stanford.edu/comp/net/iepm-bw.slac.stanford.edu/pacman-cache/','IEPM Project','http://www-iepm.slac.stanford.edu/bw/','Jerrod Williams','jerrodw@slac.standford.edu')
+	reg['LDG']       = Register(      'LDG',  'http://www.ligo.mit.edu/ldg/','LIGO Scientific Collaboration Data Grid','http://www.lsc-group.phys.uwm.edu/lscdatagrid/','Junwei Cao','jcao@ligo.mit.edu')
 	return reg
 
 class Registry(HtmlOut,IOAble):
@@ -75,6 +81,12 @@ class Registry(HtmlOut,IOAble):
 			
 		return transName
 		
+	def short(self,name0):
+		new = name0
+		for name,reg in self._registry.items():
+			if equivURL(name0,reg.url): new = name; break
+		return new
+		
 	def transPath(self,path3):
 		path = os.path.expandvars(os.path.expanduser(path3))
 		if    ':' in path: 
@@ -101,10 +113,18 @@ class Registry(HtmlOut,IOAble):
 		
 	def add(self,regEntry):
 		reason = self.addable(regEntry)
+		if not reason.ok():
+			if allow('trust-all-caches'): 
+				print 'Replacing registry entry ['+regEntry.name+']...'
+				reason = Reason()
+			elif yesno('About to replace registry entry ['+regEntry.name+']. OK?'): 
+				reason = Reason()
 		if reason.ok():
-			verbo.log('registry',regEntry.name+' added to the local registry...')
-			self._registry[regEntry.name] = regEntry
-			reason = self.save()
+			reason = ask.re('registry','OK to add ['+regEntry.name+'] to the local registry?')
+			if reason.ok():
+				verbo.log('registry',regEntry.name+' added to the local registry...')
+				self._registry[regEntry.name] = regEntry
+				reason = self.save()
 		return reason
 	def hasReg(self,regEntry):
 		reason = Reason()
@@ -130,8 +150,8 @@ class Registry(HtmlOut,IOAble):
 				del self._registry[regEntry.name]
 				verbo.log('registry',regEntry.name+' removed from the local registry...')
 				reason = self.save()
-			else:
-				reason = Reason("Registry does not contain ["+`regEntry`+"]. Can't remove.")					
+#			else:
+#				reason = Reason("Registry does not contain ["+`regEntry`+"]. Can't remove.")					
 		return reason
 		
 	def save(self,file=''): 
@@ -173,11 +193,12 @@ class Registry(HtmlOut,IOAble):
 	def display(self,indent=0):
 		keys = self._registry.keys()
 		keys.sort()
-		print '   - Cache -     - Manager -           - Email -                            - Location - '
+		print '- Name -      - Location -                                                            - Contact -          '
 		print 140*'-'
 		for key in keys:
 			reg = self._registry[key]
-			print '  ',reg.name,max(0,12-len(reg.name))*' ',reg.contactName,max(0,20-len(reg.contactName))*' ',reg.contactEmail,max(0,35-len(reg.contactEmail))*' ',reg.url
+#			print '  ',reg.name,max(0,12-len(reg.name))*' ',reg.contactName,max(0,20-len(reg.contactName))*' ',reg.contactEmail,max(0,35-len(reg.contactEmail))*' ',reg.url
+			print reg.name,max(0,12-len(reg.name))*' ',reg.url,max(0,70-len(reg.url))*' ',reg.contactName,max(0,20-len(reg.contactName))*' ',reg.contactEmail
 		print 140*'-'
 
 class LocalRegistry(Environment):
@@ -229,9 +250,11 @@ class Register(Environment):
 		w.text('registry entry: '); self.nameHtml(w); w.text(' '); self.infoHtml(w); w.text(' '); self.contactHtml(w)
 
 #-- satisfies
-	def satisfiable(self): return registry.addable(self)
-	def satisfied  (self): return registry.hasReg (self)
+#	def satisfiable(self): return registry.addable(self)
+	def satisfiable(self): return Reason()
+#	def satisfied  (self): return registry.hasReg (self)
 	def acquire    (self): return registry.add    (self)    
-	def retract    (self): return registry.remove (self)			
+#	def retract    (self): return registry.remove (self)	
+	def retract    (self): return Reason()		
 
 registry = Registry()
