@@ -20,15 +20,14 @@ class SoftLink(Environment):
 	def satisfiable(self): return Reason()
 	def acquire(self):
 		reason = Reason()
-		if os.path.exists(self.linkFrom):
-			reason = Reason("File ["+self.linkFrom+"] exists.  Can't create link.")
+		if os.path.exists(self.linkTo):
+			reason = Reason("File ["+self.linkTo+"] exists.  Can't create link.")
 		else:
 			self.linkFrom = fullpath(self.linkFrom)
 			self.linkTo   = fullpath(self.linkTo)
-#			reason = execute('ln -s '+self.linkTo+' '+self.linkFrom)
 			reason = execute('ln -s '+self.linkFrom+' '+self.linkTo)
 		return reason
-	def retract(self): return execute('rm -f '+self.linkFrom)
+	def retract(self): return execute('rm -f '+self.linkTo)
 	
 
 				
