@@ -4,6 +4,30 @@
 from Environment  import *
 from scanPorts    import *
 
+class UsePortNumber(Environment):
+	type   = 'bind a socket to port'
+	action = 'test if a socket can be bound to a given port'
+	title  = 'Bind a Socket to Port'
+	
+	def __init__(self,port):
+		self._port = port
+		
+	def equal(self,x): return self._port == x._port
+	def str(self): return `self._port`
+	
+	def satisfied  (self): return Reason(`self`+" has not been tested yet.",not self.acquired)
+	def satisfiable(self): return Reason()
+	
+	def acquire(self):
+		verbo.log('tcp','Try binding a socket to TCP port '+`self._port`+'...')
+		msg = tryServerSocket(self._port)
+		if msg == '': r = Reason()
+		else:         r = Reason(msg)
+		return r
+		
+	def retract(self): return Reason()
+	def verify (self): return self.acquire()
+
 class TCPPorts(Environment):
 	type = 'tcpPorts'
 	action = 'test TCP port range'
