@@ -35,10 +35,10 @@ class Trust:
 			if ok: self._trusted[cache] = ''
 		return ok
 		
-	def add(self,cache):
+	def add(self,cache,autotrust=0):
 		r = Reason()
 		if not cache=='trusted.caches' and not self.trusted(cache):
-			if switch('trust-all-caches') or yesno('Do you want to add ['+cache+'] to [trusted.caches]?'):
+			if switch('trust-all-caches') or allow('trust-all-caches') or yesno('Do you want to add ['+cache+'] to [trusted.caches]?'):
 				try:
 					f = open(self._trustfile,'a')
 					f.write(cache+'\n')
