@@ -3,9 +3,15 @@
 #
 #       Saul Youssef, March 2005 - rewrite to handle tar-overwrites correctly.
 #
+#   Oct 2005, removing --no-same-owner options for TeraGrid
+#
 from Base        import *
 from Environment import *
 import EnvironmentVariable,Execution
+import freedisk
+import time
+
+gnuTarOK,tarName = gnuTarFinder()
 	
 class TarSave:
 	def __init__(self):
@@ -68,6 +74,7 @@ class Untarzip(Environment):
 		self._contents = []
 		self._saved    = {}
 		self._enviro   = None
+		self._tarpause = 0
 		
 	def getSaved(self):
 		if hasattr(self,'_saved'): return self._saved
@@ -76,6 +83,14 @@ class Untarzip(Environment):
 		if hasattr(self,'_enviro'): return self._enviro
 		else:                       return None
 		
+	def getTarpath(self):
+		if hasattr(self,'_tarpath'): return self._tarpath
+		else:                        return ''
+		
+	def getTarPause(self):
+		if hasattr(self,'_tarpause'): return self._tarpause
+		else:                         return tarGetPause()
+		
 	def equal(self,u): return self._tarball==u._tarball and self._env==u._env
 	def str(self):     
 		s = self._tarball
@@ -100,25 +115,38 @@ class Untarzip(Environment):
 		else:                                self._tarpath = self._tarball[:  ]
 
 	def getContents(self):
-		verbo.log('shell-all','About to [tar --no-same-owner -t -f '+self._tarpath+']...')
-		status,output = commands.getstatusoutput('tar --no-same-owner -t -f '+self._tarpath)
-		if status!=0: status,output = commands.getstatusoutput('tar -t -f '+self._tarpath)
+		c = []
+		r = Reason()
+		if self.unzip(): z = ' -z'
+		else:            z = ''
+		
+		command = gnuTarName()+z+' -t -f '+self._tarball
+		verbo.log('shell-all','About to execute ['+command+']...')
+		r = ask.re('shell-all','OK to execute ['+command+']?')
+		if r.ok(): r = ask.re('tar','OK to extract the contents of tarball ['+os.path.basename(self._tarball)+']?')
+		if r.ok():
+			if not self.getTarPause()==0: 
+				verbo.log('tar','tarPause: sleeping for '+`self.getTarPause()`+' seconds...')
+				time.sleep(self.getTarPause())
+			status,output = commands.getstatusoutput(command)
 			
-		if status==0: return Reason(),string.split(output,'\n')
-		else:         return Reason('Error reading '+self._tarpath+': ['+output+']'),[]
+			if status==0:
+				t = string.split(output,'\n')
+				c = []
+				for tt in t:
+					if len(tt)>1 and tt[:2]=='./': c.append(tt[2:])
+					else:                          c.append(tt    )
+			else:
+				r = Reason('Error executing ['+command+'] returns status code ['+`status`+'].')
+		return r,c
 
 	def satisfied  (self): return Reason('Tarball ['+self._tarball+'] has not been untar/zipped',not self.acquired)
 	def satisfiable(self): return Reason()
 
 	def acquire(self):
-		verbo.log('tar','Untarring ['+self._tarball+']...')
-		r = Reason('Tarball ['+fullpath(self._tarball)+"] doesn't exist.",not os.path.exists(fullpath(self._tarball)))
-		if r.ok():
-			self.unzipsuff()
-			self._tarpath = fullpath(self._tarpath)
-			self._contents = []
-			self._saved    = {}
-		if r.ok() and self.unzip(): r = Execution.execute('gunzip '+self._tarball)
+		self._tarball = fullpath(self._tarball)
+		verbo.log('tar','Untarring ['+os.path.basename(self._tarball)+']...')
+		r = Reason('Tarball ['+self._tarball+"] doesn't exist.",not os.path.exists(fullpath(self._tarball)))
 		
 		if r.ok():
 			r,paths = self.getContents()
@@ -147,38 +175,57 @@ class Untarzip(Environment):
 		if r.ok(): 
 			r = Reason()
 			ver = verbo('tarfiles')
+			bad = allow('bad-tar-filenames')
 			for x in self._contents:
-				if ver: print x
-				if tail(x,'/'):
-					if os.path.exists(x):
-						if not os.path.isdir(x):
-							r = Reason('Untarring ['+self._tarball+'] would overwrite ['+x+'] with a directory.')
-							break
-						_tarSave.add(self._tarpath,x)
-				else:
-					if os.path.exists(x):
-						if os.path.isdir(x):
-							r = Reason('Untarring ['+self._tarball+'] would overwrite directory ['+x+'] with a file.')
-						else:
-							verbo.log('tarfiles','Untarring ['+self._tarball+'] will overwrite ['+x+'] saving...')
-							r = ask.re('tar-overwrite','OK to overwrite ['+x+'] untarring ['+self._tarball+']?')
-							if r.ok():
-								if not allow('tar-overwrite'): print 'WARNING: Untarring tarball ['+self._tarball+'] will overwrite ['+x+']...'
-								try:
-									f = open(x,'r')
-									xx = f.read()
-									f.close()
-									self._saved[x] = xx
-								except (IOError,OSError):
-									r = Reason('Error reading ['+x+'].')
-						_tarSave.add(self._tarpath,x)
-				if not r.ok(): break
+				if 0 and not bad and contains(x,'::'):
+				   	r = Reason('File ['+x+'] from tarball ['+self._tarball+'] contains a file name with a non-standard character (see -allow to ignore).')
+					break
+			if r.ok():
+				for x in self._contents:
+					if ver: print x
+					if tail(x,'/'):
+						if os.path.exists(x):
+							if not os.path.isdir(x):
+								r = Reason('Untarring ['+self._tarball+'] would overwrite ['+x+'] with a directory.')
+								break
+							_tarSave.add(self.getTarpath(),x)
+					else:
+						if os.path.exists(x):
+							if os.path.isdir(x):
+								r = Reason('Untarring ['+self._tarball+'] would overwrite directory ['+x+'] with a file.')
+							else:
+								verbo.log('tarfiles','Untarring ['+self._tarball+'] will overwrite ['+x+'] saving...')
+								r = ask.re('tar-overwrite','OK to overwrite ['+x+'] untarring ['+self._tarball+']?')
+								if r.ok():
+									if not allow('tar-overwrite'): print 'WARNING: Untarring tarball ['+self._tarball+'] will overwrite ['+x+']...'
+									try:
+										f = open(x,'r')
+										xx = f.read()
+										f.close()
+										self._saved[x] = xx
+									except (IOError,OSError):
+										r = Reason('Error reading ['+x+'].')
+							_tarSave.add(self.getTarpath(),x)
+					if not r.ok(): break
 
 		if r.ok(): 
-			r =                Execution.execute('tar --no-same-owner -xf '+os.path.basename(self._tarpath))
-			if not r.ok(): r = Execution.execute('tar                 -xf '+os.path.basename(self._tarpath))
-			if r.ok(): removeFile(self._tarpath)
-			
+			r = ask.re('tar','OK to untar ['+os.path.basename(self._tarball)+']?')
+			if r.ok():
+				if self.unzip(): z = ' -z'
+				else:            z = ''
+				if not os.path.exists(os.path.basename(self._tarball)): r = Reason("Tarball ["+os.path.basename(self._tarball)+"] does not exist in ["+os.getcwd()+"].  Can't untar.")
+				if debug('tar'): 
+					print self._tarball
+					os.system('ls')
+				if r.ok():
+				        if not self.getTarPause()==0: 
+						verbo.log('tar','tarPause: sleeping for '+`self.getTarPause()`+' seconds...')
+						time.sleep(self.getTarPause())
+#					r = Execution.execute(gnuTarName()+z+' -xf '+os.path.basename(self._tarball))
+					if gnuTarOK: r = Execution.execute(gnuTarName()+' --no-same-owner '+z+' -xf '+os.path.basename(self._tarball))
+					else:        r = Execution.execute(gnuTarName()+                    z+' -xf '+os.path.basename(self._tarball))
+					if not freedisk.enoughFreeDisk(100).ok(): r = freedisk.enoughFreeDisk(100)
+					removeFile(os.path.basename(self._tarball))
 		if r.ok():
 			if not self._env=='':
 				if len(self._contents)==0:
@@ -199,21 +246,23 @@ class Untarzip(Environment):
 		for x in self._contents:
 			if os.path.exists(x):
 				if tail(x,'/'): 
-					_tarSave.rem(self._tarpath,x)
+					_tarSave.rem(self.getTarpath(),x)
 					if _tarSave.num(x)==0: 
 #-- for speed
 						r = Execution.execute('rm -r -f "'+x+'"')
 						if not r.ok():
-							r = Execution.execute('chmod -R a+w "'+x+'"')
-							if r.ok(): r = Execution.execute('rm -r -f "'+x+'"')
+							r2 = Execution.execute('chmod -R a+w "'+x+'"')
+							if r2.ok(): r2 = Execution.execute('rm -r -f "'+x+'"')
+							if r2.ok(): r = r2
 				else:
-					saved = _tarSave.has(self._tarpath,x)
-					_tarSave.rem(self._tarpath,x)
+					saved = _tarSave.has(self.getTarpath(),x)
+					_tarSave.rem(self.getTarpath(),x)
 					if saved and _tarSave.num(x)==0: 
 						r = Execution.execute('rm -f "'+x+'"')
 						if not r.ok():
-							r = Execution.execute('chmod a+w "'+x+'"')
-							if r.ok(): r = Execution.execute('rm -f "'+x+'"')
+							r2 = Execution.execute('chmod a+w "'+x+'"')
+							if r2.ok(): r2 = Execution.execute('rm -f "'+x+'"')
+							if r2.ok(): r = r2
 						if r.ok():
 							if self._saved.has_key(x):
 								r = ask.re('tar-overwrite','OK to restore ['+x+'] previously overwritten by ['+self._tarball+']?')
@@ -230,7 +279,19 @@ class Untarzip(Environment):
 		return r
 		
 	def verify(self):
-		return allReason(self._contents,lambda x: Reason('Untarred file ['+x+'] from ['+self._tarball+'] is missing.',not os.path.exists(x)))
+		v = verbo('tarfiles')
+		if v:
+			if len(self._contents)>1000: m = Meter(len(self._contents),'Verifying untarred content of ['+self._tarball+']...')
+			else:  print 'Verifying untarred content of ['+self._tarball+']...'
+			r = AllReason()
+			count = 0
+			for x in self._contents:
+				count = count + 1
+				if len(self._contents)>1000: m.meter(count)
+				if not os.path.exists(x): r.append(Reason('Untarred file ['+x+'] from ['+self._tarball+'] is missing.'))
+		else:
+			r = allReason(self._contents,lambda x: Reason('Untarred file ['+x+'] from ['+self._tarball+'] is missing.',not os.path.exists(x)))
+		return r
 
 class UntarzipDeletable(Untarzip):
 	type   = 'untar vdt'
