@@ -15,7 +15,8 @@ def lock():
 			print 'Error reading Pacman directory area.'
 			sys.exit(1)
 		if len(lines)>0 and len(lines[0])>1:
-			print 'Installation ['+pac_anchor+'] is currently in use by ['+lines[0][:-1]+"]."
+			print 'Installation ['+pac_anchor+'] is currently locked by ['+lines[0][:-1]+"]."
+			print 'Use [% pacman -clear-lock] to override.'
 			sys.exit(1)
 		else:
 			print 'Error reading Pacman directory area.'
