@@ -14,12 +14,14 @@ def switchFinalize():
 	browser.save()
 	allow.save()
 	upOptions.save()
-	domainMode.save()
+#	domainMode.save()
 	displayMode.save()
 	ask.save()
 	allow.save()
 	
 	pre,plat = switchpar('pretend-platform')
+	if not pre and len(switchItems('pretend-platform'))>0:
+		pre,plat = 1,switchItems('pretend-platform')[0]
 	if pre:
 		ppl = os.path.join(pac_anchor,pacmanDir,'ppl')
 		try:
